﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Input;
using B3.PluginAPIKit;

namespace PluginApiDetails
{
    public class PluginApiDetails : IPlugin
    {
        private IHost mHost;
        private bool mActivated;
        private INavContext mNav;
        private IEditorContext mEditorContext;

        public void Execute()
        {
            mActivated = true;
            mNav = mHost.GetNavContext();
            mEditorContext = mHost.GetEditorContext();
            CreatePluginApiDetailsToolstripAndRibbon();
        }

        public string GetAuthor()
        {
            return "Thomas Tregner";
        }

        public string GetDescription()
        {
            return "Button clicks show values returned by certain API calls.";
        }

        public string GetName()
        {
            return "Plugin API Details";
        }

        public string GetVersion()
        {
            return "0.1";
        }

        public void Initialize(IHost host)
        {
            mHost = host;
        }

        public bool IsActivated
        {
            get { return mActivated; }
        }

        public void Stop()
        {

            MessageBox.Show(GetName() + " deactivated.");
            mHost.Dispose();
            mActivated = false;
        }

        private void CreatePluginApiDetailsToolstripAndRibbon()
        {
            ///Toolstrips and ribbons tabs are created separately.
            ///Toolstrip
            ICustomToolstrip toolstripApiDetails = mNav.CreateCustomToolstrip("API Details");
            toolstripApiDetails.AddToolStripButton("API Details", null, button_click_api_details);

            //Ribbon
            IRibbon ribbon = mNav.GetRibbon();
            IRibbonTab tab = ribbon.AddNewRibbonTab("API Details");
            IRibbonGroup group = tab.AddNewRibbonGroup("API Details");
            group.AddRibbonButton("API Details", new ButtonCommand(mEditorContext), null, null, RibbonIconSize.Collapsed, null, null);

        }

        private static void apiDetailsButtonsActions(IEditorContext pEditorContext)
        {
            IDocument activeDocument = pEditorContext.GetActiveDocument();
            
            ISelection selection;
            String sourceUrl = "";
            String backColor = "";
            String cursorPosition = "";
            String fontfamily = "";
            String fontSize = "";
            String foreColor = "";
            String style = "";
            List<KeyValuePair<string, string>> documents = new List<KeyValuePair<string, string>>();
            String documentText = "";
            String documentXml = "";
            string selectionText = "";

            if (activeDocument != null)
            {
                selection = activeDocument.Selection;
                sourceUrl = activeDocument.GetSourceUrl();
                backColor = selection.GetBackColor().ToString();
                cursorPosition = activeDocument.GetCursorPosition().ToString();
                fontfamily = selection.GetFontFamily();
                fontSize = selection.GetFontSize().ToString();
                foreColor = selection.GetForeColor().ToString();
                style = selection.GetStyle();
                documentText = activeDocument.GetDocumentText();
                documentXml = activeDocument.GetDocumentXml();
                selectionText = selection.GetText();

                foreach (IDocument doc in pEditorContext.GetDocuments())
                {
                    //will fail if the document doesn't have 10 characters
                    documents.Add(new KeyValuePair<string, string>(doc.GetSourceUrl(), doc.GetDocumentText().Substring(0, 10)));
                }
            }

            Details DetailsScreen = new Details(sourceUrl,backColor,cursorPosition,
                                                fontfamily,fontSize,foreColor,style,
                                                documents,documentText, documentXml,
                                                selectionText);
            DetailsScreen.Show();
        }

        private void button_click_api_details(object sender, EventArgs e)
        {
            apiDetailsButtonsActions(mEditorContext);
        }

        private class ButtonCommand : ICommand
        {
            private IEditorContext passEditorContext;

            public ButtonCommand(IEditorContext pEditorContext)
            {
                passEditorContext = pEditorContext;
            }

            public bool CanExecute(object parameter)
            {
                return true;
            }

            public event EventHandler CanExecuteChanged;

            public void Execute(object parameter)
            {
                apiDetailsButtonsActions(passEditorContext);
            }
        }

    }
}
